@@ -33,20 +33,34 @@ use JSON;
 use Scalar::Util;
 
 my %hierarchy = (
+    'AccentElement' => 'MathElement',
     'AddLayoutSlide' => 'Task',
     'AddMasterSlide' => 'Task',
     'AddShape' => 'Task',
     'AddSlide' => 'Task',
+    'ArcToPathSegment' => 'PathSegment',
+    'ArrayElement' => 'MathElement',
+    'BarElement' => 'MathElement',
     'Base64InputFile' => 'InputFile',
+    'BlockElement' => 'MathElement',
+    'BorderBoxElement' => 'MathElement',
+    'BoxElement' => 'MathElement',
     'BubbleSeries' => 'Series',
+    'ClosePathSegment' => 'PathSegment',
     'ColorScheme' => 'ResourceBase',
+    'CubicBezierToPathSegment' => 'PathSegment',
+    'DelimiterElement' => 'MathElement',
     'Document' => 'ResourceBase',
     'DocumentProperties' => 'ResourceBase',
     'DocumentProperty' => 'ResourceBase',
     'FileVersion' => 'StorageFile',
     'FontScheme' => 'ResourceBase',
     'FormatScheme' => 'ResourceBase',
+    'FractionElement' => 'MathElement',
+    'FunctionElement' => 'MathElement',
+    'GifExportOptions' => 'ExportOptions',
     'GradientFill' => 'FillFormat',
+    'GroupingCharacterElement' => 'MathElement',
     'HeaderFooter' => 'ResourceBase',
     'Html5ExportOptions' => 'ExportOptions',
     'HtmlExportOptions' => 'ExportOptions',
@@ -55,9 +69,15 @@ my %hierarchy = (
     'Images' => 'ResourceBase',
     'LayoutSlide' => 'ResourceBase',
     'LayoutSlides' => 'ResourceBase',
+    'LeftSubSuperscriptElement' => 'MathElement',
+    'LimitElement' => 'MathElement',
+    'LineToPathSegment' => 'PathSegment',
     'MasterSlide' => 'ResourceBase',
     'MasterSlides' => 'ResourceBase',
+    'MatrixElement' => 'MathElement',
     'Merge' => 'Task',
+    'MoveToPathSegment' => 'PathSegment',
+    'NaryOperatorElement' => 'MathElement',
     'NoFill' => 'FillFormat',
     'NotesSlide' => 'ResourceBase',
     'NotesSlideHeaderFooter' => 'ResourceBase',
@@ -76,6 +96,8 @@ my %hierarchy = (
     'Portions' => 'ResourceBase',
     'PptxExportOptions' => 'ExportOptions',
     'ProtectionProperties' => 'ResourceBase',
+    'QuadraticBezierToPathSegment' => 'PathSegment',
+    'RadicalElement' => 'MathElement',
     'RemoveShape' => 'Task',
     'RemoveSlide' => 'Task',
     'ReorderSlide' => 'Task',
@@ -83,6 +105,7 @@ my %hierarchy = (
     'RequestInputFile' => 'InputFile',
     'ResetSlide' => 'Task',
     'ResponseOutputFile' => 'OutputFile',
+    'RightSubSuperscriptElement' => 'MathElement',
     'Save' => 'Task',
     'SaveShape' => 'Task',
     'SaveSlide' => 'Task',
@@ -100,14 +123,18 @@ my %hierarchy = (
     'Slides' => 'ResourceBase',
     'SolidFill' => 'FillFormat',
     'SplitDocumentResult' => 'ResourceBase',
+    'SubscriptElement' => 'MathElement',
+    'SuperscriptElement' => 'MathElement',
     'SvgExportOptions' => 'ExportOptions',
     'SwfExportOptions' => 'ExportOptions',
+    'TextElement' => 'MathElement',
     'TextItems' => 'ResourceBase',
     'Theme' => 'ResourceBase',
     'TiffExportOptions' => 'ExportOptions',
     'UpdateBackground' => 'Task',
     'UpdateShape' => 'Task',
     'ViewProperties' => 'ResourceBase',
+    'XamlExportOptions' => 'ExportOptions',
     'XpsExportOptions' => 'ExportOptions',
     'BoxAndWhiskerSeries' => 'OneValueSeries',
     'BubbleChartDataPoint' => 'ScatterChartDataPoint',
@@ -158,8 +185,11 @@ my %determiners = (
     'FillFormat' => {  },
     'FillOverlayEffect' => {  },
     'FontSet' => {  },
+    'GeometryPath' => {  },
+    'GeometryPaths' => {  },
     'GlowEffect' => {  },
     'GradientFillStop' => {  },
+    'Hyperlink' => {  },
     'IShapeExportOptions' => {  },
     'ImageExportFormat' => {  },
     'InnerShadowEffect' => {  },
@@ -169,6 +199,8 @@ my %determiners = (
     'Legend' => {  },
     'LightRig' => {  },
     'LineFormat' => {  },
+    'MathElement' => {  },
+    'MathParagraph' => {  },
     'MergingSource' => {  },
     'NormalViewRestoredProperties' => {  },
     'NotesSlideExportFormat' => {  },
@@ -176,6 +208,7 @@ my %determiners = (
     'OrderedMergeRequest' => {  },
     'OuterShadowEffect' => {  },
     'OutputFile' => {  },
+    'PathSegment' => {  },
     'Pipeline' => {  },
     'PlotArea' => {  },
     'PresentationToMerge' => {  },
@@ -204,20 +237,34 @@ my %determiners = (
     'Task' => {  },
     'TextItem' => {  },
     'ThreeDFormat' => {  },
+    'AccentElement' => { 'Type' => 'Accent', },
     'AddLayoutSlide' => { 'Type' => 'AddLayoutSlide', },
     'AddMasterSlide' => { 'Type' => 'AddMasterSlide', },
     'AddShape' => { 'Type' => 'AddShape', },
     'AddSlide' => { 'Type' => 'AddSlide', },
+    'ArcToPathSegment' => { 'Type' => 'ArcTo', },
+    'ArrayElement' => { 'Type' => 'Array', },
+    'BarElement' => { 'Type' => 'Bar', },
     'Base64InputFile' => { 'Type' => 'Base64', },
+    'BlockElement' => { 'Type' => 'Block', },
+    'BorderBoxElement' => { 'Type' => 'BorderBox', },
+    'BoxElement' => { 'Type' => 'Box', },
     'BubbleSeries' => { 'DataPointType' => 'Bubble', },
+    'ClosePathSegment' => { 'Type' => 'Close', },
     'ColorScheme' => {  },
+    'CubicBezierToPathSegment' => { 'Type' => 'CubicBezierTo', },
+    'DelimiterElement' => { 'Type' => 'Delimiter', },
     'Document' => {  },
     'DocumentProperties' => {  },
     'DocumentProperty' => {  },
     'FileVersion' => {  },
     'FontScheme' => {  },
     'FormatScheme' => {  },
+    'FractionElement' => { 'Type' => 'Fraction', },
+    'FunctionElement' => { 'Type' => 'Function', },
+    'GifExportOptions' => { 'Format' => 'gif', },
     'GradientFill' => { 'Type' => 'Gradient', },
+    'GroupingCharacterElement' => { 'Type' => 'GroupingCharacter', },
     'HeaderFooter' => {  },
     'Html5ExportOptions' => { 'Format' => 'html5', },
     'HtmlExportOptions' => { 'Format' => 'html', },
@@ -226,9 +273,15 @@ my %determiners = (
     'Images' => {  },
     'LayoutSlide' => {  },
     'LayoutSlides' => {  },
+    'LeftSubSuperscriptElement' => { 'Type' => 'LeftSubSuperscriptElement', },
+    'LimitElement' => { 'Type' => 'Limit', },
+    'LineToPathSegment' => { 'Type' => 'LineTo', },
     'MasterSlide' => {  },
     'MasterSlides' => {  },
+    'MatrixElement' => { 'Type' => 'Matrix', },
     'Merge' => { 'Type' => 'Merge', },
+    'MoveToPathSegment' => { 'Type' => 'MoveTo', },
+    'NaryOperatorElement' => { 'Type' => 'NaryOperator', },
     'NoFill' => { 'Type' => 'NoFill', },
     'NotesSlide' => {  },
     'NotesSlideHeaderFooter' => {  },
@@ -247,6 +300,8 @@ my %determiners = (
     'Portions' => {  },
     'PptxExportOptions' => { 'Format' => 'pptx', },
     'ProtectionProperties' => {  },
+    'QuadraticBezierToPathSegment' => { 'Type' => 'QuadBezierTo', },
+    'RadicalElement' => { 'Type' => 'Radical', },
     'RemoveShape' => { 'Type' => 'RemoveShape', },
     'RemoveSlide' => { 'Type' => 'RemoveSlide', },
     'ReorderSlide' => { 'Type' => 'ReoderSlide', },
@@ -254,6 +309,7 @@ my %determiners = (
     'RequestInputFile' => { 'Type' => 'Request', },
     'ResetSlide' => { 'Type' => 'ResetSlide', },
     'ResponseOutputFile' => { 'Type' => 'Response', },
+    'RightSubSuperscriptElement' => { 'Type' => 'RightSubSuperscriptElement', },
     'Save' => { 'Type' => 'Save', },
     'SaveShape' => { 'Type' => 'SaveShape', },
     'SaveSlide' => { 'Type' => 'SaveSlide', },
@@ -271,14 +327,18 @@ my %determiners = (
     'Slides' => {  },
     'SolidFill' => { 'Type' => 'Solid', },
     'SplitDocumentResult' => {  },
+    'SubscriptElement' => { 'Type' => 'SubscriptElement', },
+    'SuperscriptElement' => { 'Type' => 'SuperscriptElement', },
     'SvgExportOptions' => { 'Format' => 'svg', },
     'SwfExportOptions' => { 'Format' => 'swf', },
+    'TextElement' => { 'Type' => 'Text', },
     'TextItems' => {  },
     'Theme' => {  },
     'TiffExportOptions' => { 'Format' => 'tiff', },
     'UpdateBackground' => { 'Type' => 'UpdateBackground', },
     'UpdateShape' => { 'Type' => 'UpdateShape', },
     'ViewProperties' => {  },
+    'XamlExportOptions' => { 'Format' => 'xaml', },
     'XpsExportOptions' => { 'Format' => 'xps', },
     'BoxAndWhiskerSeries' => { 'DataPointType' => 'OneValue', },
     'BubbleChartDataPoint' => {  },
@@ -301,6 +361,19 @@ my %determiners = (
     'VideoFrame' => { 'Type' => 'VideoFrame', },
 );
 
+sub is_subclass {
+    my ($self, $subclass, $class) = @_;
+    if ($subclass eq $class) {
+        return 1;
+    }
+    for(keys %hierarchy) {
+        if ($hierarchy{$_} eq $class && $self->is_subclass($subclass, $_)) {
+            return 1;
+        }
+    }
+    return 0;
+}
+
 sub get_class_name {
     my ($self, $name, $data) = @_;
     my $descendant = $self->get_subclass_name($name, $data);
